#include "cbase.h"
#include "ai_network.h"
#include "ai_networkmanager.h"
#include "sdk_mergingnodeweight.h"

// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

ConVar nodeweight_merge_factor( "nodeweight_merge_factor", "0.5");

LINK_ENTITY_TO_CLASS( sdk_mergingnodeweight, CMergingNodeWeight );

// Start of our data description for the class
BEGIN_DATADESC( CMergingNodeWeight )
	
END_DATADESC()

void CMergingNodeWeight::Initialize()
{
	Buffer = CUtlVector<Vector>(0, g_pBigAINet->NumNodes());
	BaseClass::Initialize();
}


// Updates the weights over time
void CMergingNodeWeight::Update()
{
	float currentFactor = nodeweight_merge_factor.GetFloat();
	float bufferFactor = 1 - currentFactor;
	Vector resetValue = Vector(0.0,0.0,0.0);

	for(int i=CurrentWeights.Count()-1;i>=0;i--) 
	{
		Buffer[i] = CurrentWeights[i] * currentFactor + Buffer[i] * bufferFactor;
		CurrentWeights[i] = resetValue;
	}
}

Vector CMergingNodeWeight::GetNodeWeight(int nodeIndex) 
{
	return CurrentWeights[nodeIndex] * nodeweight_merge_factor.GetFloat() + Buffer[nodeIndex] * (1 - nodeweight_merge_factor.GetFloat());
}


void CMergingNodeWeight::Reset()
{
	BaseClass::Reset();

	Buffer.RemoveAll();

	Vector a = Vector(0.0,0.0,0.0);
	for(int i=0;i<g_pBigAINet->NumNodes();i++) 
	{
		Buffer.AddToTail(a);
	}
}
