//========= Copyright  1996-2005, Valve Corporation, All rights reserved. ============//
// This is a skeleton file for use when creating a new 
// NPC. Copy and rename this file for the new
// NPC and add the copy to the build.
//
// Leave this file in the build until we ship! Allowing 
// this file to be rebuilt with the rest of the game ensures
// that it stays up to date with the rest of the NPC code.
//
// Replace occurances of CBugNPC with the new NPC's
// classname. Don't forget the lower-case occurance in 
// LINK_ENTITY_TO_CLASS()
//
//
// ASSUMPTIONS MADE:
//
// You're making a character based on CAI_BaseNPC. If this 
// is not true, make sure you replace all occurances
// of 'CAI_BaseNPC' in this file with the appropriate 
// parent class.
//
// You're making a human-sized NPC that walks.
//
//=============================================================================//
#include "cbase.h"
#include "ai_default.h"
#include "ai_task.h"
#include "ai_schedule.h"
#include "ai_hull.h"
#include "ai_network.h"
#include "ai_networkmanager.h"
#include "ai_node.h"
#include "soundent.h"
#include "gib.h"
#include "te_effect_dispatch.h"
#include "sp2_brainbug.h"

#include "game.h"
#include "npcevent.h"
#include "entitylist.h"
#include "activitylist.h"
#include "ai_basenpc.h"
#include "npc_antlion.h"
#include "hl2_shareddefs.h"
#include "engine/IEngineSound.h"
#include "physics_prop_ragdoll.h"
#include "sdk_ourrules.h"
#include "sp2_bugpoi.h"
#include "HL2_PLayer.h"
#include "sp2_bug.h"
#include "decals.h"
#include "antlion_dust.h"
// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

extern int AE_ANTLION_WALK_FOOTSTEP;
extern int AE_ANTLION_MELEE_HIT1;
extern int AE_ANTLION_MELEE_HIT2;
extern int AE_ANTLION_MELEE_POUNCE;
extern int AE_ANTLION_FOOTSTEP_SOFT;
extern int AE_ANTLION_FOOTSTEP_HEAVY;
extern int AE_ANTLION_START_JUMP;
extern int AE_ANTLION_BURROW_IN;
extern int AE_ANTLION_BURROW_OUT;
extern int AE_ANTLION_VANISH;
extern int AE_ANTLION_OPEN_WINGS;
extern int AE_ANTLION_CLOSE_WINGS;
extern int AE_ANTLION_MELEE1_SOUND;
extern int AE_ANTLION_MELEE2_SOUND;

extern int ACT_ANTLION_BURROW_IN;
extern int ACT_ANTLION_BURROW_OUT;
extern int ACT_ANTLION_BURROW_IDLE;

ConVar	sp2_bug_always_gib( "sp2_bug_always_gib", "0" );
ConVar	sp2_turret_dmg_multiplier( "sp2_turret_dmg_multiplier", "1" );


//Attack range definitions
#define	BUG_MELEE1_RANGE		100.0f

LINK_ENTITY_TO_CLASS( sp2_bug, CBugNPC );

//---------------------------------------------------------
// Save/Restore
//---------------------------------------------------------
BEGIN_DATADESC( CBugNPC )
	DEFINE_FIELD( m_vecArmorColor, FIELD_VECTOR ),
	DEFINE_KEYFIELD( m_vecArmorColor, FIELD_VECTOR, "ArmorColor" ),
END_DATADESC()


//-----------------------------------------------------------------------------
// Purpose: Initialize the custom schedules
// Input  :
// Output :
//-----------------------------------------------------------------------------

AI_BEGIN_CUSTOM_NPC( sp2_bug, CBugNPC )
	
	DECLARE_CONDITION( COND_BUG_REACHED_GOAL )
		
	DECLARE_TASK( TASK_FIND_BUG_GOAL )
	DECLARE_TASK(TASK_STAY_BURROWED)
	DECLARE_ACTIVITY( ACT_ANTLION_BURROW_IN )
	DECLARE_ACTIVITY( ACT_ANTLION_BURROW_OUT )
	DECLARE_ACTIVITY( ACT_ANTLION_BURROW_IDLE )
	
	//Events
	DECLARE_ANIMEVENT( AE_ANTLION_WALK_FOOTSTEP )
	DECLARE_ANIMEVENT( AE_ANTLION_MELEE_HIT1 )
	DECLARE_ANIMEVENT( AE_ANTLION_MELEE_HIT2 )
	DECLARE_ANIMEVENT( AE_ANTLION_MELEE_POUNCE )
	DECLARE_ANIMEVENT( AE_ANTLION_FOOTSTEP_SOFT )
	DECLARE_ANIMEVENT( AE_ANTLION_FOOTSTEP_HEAVY )
	DECLARE_ANIMEVENT( AE_ANTLION_START_JUMP )
	DECLARE_ANIMEVENT( AE_ANTLION_BURROW_IN )
	DECLARE_ANIMEVENT( AE_ANTLION_BURROW_OUT )
	DECLARE_ANIMEVENT( AE_ANTLION_VANISH )
	DECLARE_ANIMEVENT( AE_ANTLION_OPEN_WINGS )
	DECLARE_ANIMEVENT( AE_ANTLION_CLOSE_WINGS )
	DECLARE_ANIMEVENT( AE_ANTLION_MELEE1_SOUND )
	DECLARE_ANIMEVENT( AE_ANTLION_MELEE2_SOUND )
	
	DEFINE_SCHEDULE
	(
		SCHED_BUG_UNBURROW,

		"	Tasks"
		"		TASK_STOP_MOVING				0"
		"		TASK_PLAY_SEQUENCE		ACTIVITY:ACT_ANTLION_BURROW_OUT"
		""
		"	Interrupts"
		"		COND_TASK_FAILED"
	)
	DEFINE_SCHEDULE
    (
		SCHED_BUG_BURROW,

		"	Tasks"
		"		TASK_STOP_MOVING				0"
		"		TASK_PLAY_SEQUENCE		ACTIVITY:ACT_ANTLION_BURROW_IN"
		"		TASK_STAY_BURROWED		0"
		""
		"	Interrupts"
		"		COND_TASK_FAILED"
	)
	
	DEFINE_SCHEDULE
	(
	SCHED_BUG_BURROWED,

	"	Tasks"
	"		TASK_STAY_BURROWED	0"
	""
	"	Interrupts"
	"		COND_TASK_FAILED"
	)

	DEFINE_SCHEDULE
	(
		SCHED_RUN_TO_GOAL,
		"	Tasks"
		"		TASK_SET_TOLERANCE_DISTANCE		48"
		"		TASK_SET_ROUTE_SEARCH_TIME		3"	// Spend 3 seconds trying to build a path if stuck
		"		TASK_FIND_BUG_GOAL	0"
		"		TASK_RUN_PATH					0"
		"		TASK_WAIT_FOR_MOVEMENT			0"
		""
		"	Interrupts"
		"		COND_NEW_ENEMY"
		"		COND_LIGHT_DAMAGE"
		"		COND_HEAVY_DAMAGE"
		"		COND_HEAR_PLAYER"
		"		COND_HEAR_DANGER"
	)

	DEFINE_SCHEDULE
	(
		SCHED_WALK_TO_GOAL,
		"	Tasks"
		"		TASK_SET_TOLERANCE_DISTANCE		48"
		"		TASK_SET_ROUTE_SEARCH_TIME		3"	// Spend 3 seconds trying to build a path if stuck
		"		TASK_FIND_BUG_GOAL	0"
		"		TASK_WALK_PATH					0"
		"		TASK_WAIT_FOR_MOVEMENT			0"
		""
		"	Interrupts"
		"		COND_NEW_ENEMY"
		"		COND_LIGHT_DAMAGE"
		"		COND_HEAVY_DAMAGE"
		"		COND_HEAR_PLAYER"
		"		COND_HEAR_DANGER"

	)
	DEFINE_SCHEDULE
	(
		SCHED_BUG_IDLE,

		"	Tasks"
		"		TASK_STOP_MOVING		0"
		"		TASK_SET_ACTIVITY		ACTIVITY:ACT_IDLE"
		"		TASK_WAIT				5"
		"		TASK_WAIT_PVS			0"
		""
		"	Interrupts"
		"		COND_NEW_ENEMY"
		"		COND_SEE_FEAR"
		"		COND_LIGHT_DAMAGE"
		"		COND_HEAVY_DAMAGE"
		"		COND_SMELL"
		"		COND_PROVOKED"
		"		COND_GIVE_WAY"
		"		COND_HEAR_PLAYER"
		"		COND_HEAR_DANGER"
		"		COND_HEAR_COMBAT"
		"		COND_HEAR_BULLET_IMPACT"
		"		COND_IDLE_INTERRUPT"
	)

AI_END_CUSTOM_NPC()


//-----------------------------------------------------------------------------
// Purpose: 
//
//
//-----------------------------------------------------------------------------
void CBugNPC::Precache( void )
{
	PrecacheModel( "models/antlion.mdl" );
	PrecacheModel( "models/gibs/antlion_gib_large_1.mdl" );
	PrecacheModel( "models/gibs/antlion_gib_large_2.mdl" );
	PrecacheModel( "models/gibs/antlion_gib_large_3.mdl" );

	PrecacheModel( "models/gibs/antlion_gib_medium_1.mdl" );
	PrecacheModel( "models/gibs/antlion_gib_medium_2.mdl" );
	PrecacheModel( "models/gibs/antlion_gib_medium_3.mdl" );

	PrecacheModel( "models/gibs/antlion_gib_small_1.mdl" );
	PrecacheModel( "models/gibs/antlion_gib_small_2.mdl" );
	PrecacheModel( "models/gibs/antlion_gib_small_3.mdl" );


	PrecacheScriptSound( "NPC_Antlion.RunOverByVehicle" );
	PrecacheScriptSound( "NPC_Antlion.MeleeAttack" );
	m_hFootstep = PrecacheScriptSound( "NPC_Antlion.Footstep" );
	PrecacheScriptSound( "NPC_Antlion.BurrowIn" );
	PrecacheScriptSound( "NPC_Antlion.BurrowOut" );
	PrecacheScriptSound( "NPC_Antlion.FootstepSoft" );
	PrecacheScriptSound( "NPC_Antlion.FootstepHeavy" );
	PrecacheScriptSound( "NPC_Antlion.MeleeAttackSingle" );
	PrecacheScriptSound( "NPC_Antlion.MeleeAttackDouble" );
	PrecacheScriptSound( "NPC_Antlion.Distracted" );
	PrecacheScriptSound( "NPC_Antlion.Idle" );
	PrecacheScriptSound( "NPC_Antlion.Pain" );
	PrecacheScriptSound( "NPC_Antlion.Land" );
	PrecacheScriptSound( "NPC_Antlion.WingsOpen" );
	PrecacheScriptSound( "NPC_Antlion.LoopingAgitated" );
	PrecacheScriptSound( "NPC_Antlion.Distracted" );
	PrecacheScriptSound( "NPC_Antlion.OMG" );
	BaseClass::Precache();
}

		ConVar	sp2_bug_initial_health( "sp2_bug_initial_health", "10" );

//-----------------------------------------------------------------------------
// Purpose: 
//
//
//-----------------------------------------------------------------------------
void CBugNPC::Spawn( void )
{
	Precache();

	SetModel( "models/antlion.mdl" );
	SetHullType(HULL_MEDIUM);
	SetHullSizeNormal();
	SetDefaultEyeOffset();
	
	SetNavType( NAV_GROUND );

	SetSolid( SOLID_BBOX );
	AddSolidFlags( FSOLID_NOT_STANDABLE );
	SetMoveType( MOVETYPE_STEP );
	SetBloodColor( BLOOD_COLOR_YELLOW );
	
	SetCollisionGroup( HL2COLLISION_GROUP_ANTLION );

	m_iHealth			= sp2_bug_initial_health.GetInt();
	m_flFieldOfView		= 0.65;
	m_NPCState			= NPC_STATE_NONE;

	CapabilitiesClear();
	CapabilitiesAdd( bits_CAP_MOVE_GROUND | bits_CAP_INNATE_MELEE_ATTACK1 );

	//m_nSkin = random->RandomInt( 0, ANTLION_SKIN_COUNT-1 );

	// Set colored damage resistance:
	// Double damage from red, half damage from green, normal damage from blue..
	//m_vecArmorColor.Init(1.0, 1.0, 1.0);
	m_vecAccumulatedDamage.Init(0,0,0);

	//CapabilitiesAdd( bits_CAP_NONE );

	NPCInit();


	AddEffects( EF_NODRAW );
	AddFlag( FL_NOTARGET );
	m_spawnflags |= SF_NPC_GAG;
	//AddSolidFlags( FSOLID_NOT_SOLID );
	//m_takedamage	= DAMAGE_NO;

	SetState( NPC_STATE_IDLE );
	SetActivity( (Activity) ACT_ANTLION_BURROW_IDLE );
	SetSchedule( SCHED_BUG_UNBURROW );

}

void CBugNPC::SetBurrowed(bool value) {
}


// Number of times the antlions will attempt to generate a random chase position
#define NUM_CHASE_POSITION_ATTEMPTS		3

//-----------------------------------------------------------------------------
// Purpose: 
// Input  : &targetPos - 
//			&result - 
// Output : Returns true on success, false on failure.
//-----------------------------------------------------------------------------
bool CBugNPC::FindChasePosition( const Vector &targetPos, Vector &result )
{
	if ( HasSpawnFlags( SF_ANTLION_USE_GROUNDCHECKS ) == true )
	{
		result = targetPos;
		return true;
	}

	Vector runDir = ( targetPos - GetAbsOrigin() );
	VectorNormalize( runDir );

	Vector	vRight, vUp;
	VectorVectors( runDir, vRight, vUp );

	for ( int i = 0; i < NUM_CHASE_POSITION_ATTEMPTS; i++ )
	{
		result	= targetPos;
		result += -runDir * random->RandomInt( 64, 128 );
		result += vRight * random->RandomInt( -128, 128 );

		//FIXME: We need to do a more robust search here
		// Find a ground position and try to get there
		if ( GetGroundPosition( result, result ) )
			return true;
	}

	return false;
}

//-----------------------------------------------------------------------------
// Purpose: 
// Input  : &testPos - 
//-----------------------------------------------------------------------------
bool CBugNPC::GetGroundPosition( const Vector &testPos, Vector &result )
{
	// Trace up to clear the ground
	trace_t	tr;
	AI_TraceHull( testPos, testPos + Vector( 0, 0, 64 ), NAI_Hull::Mins( GetHullType() ), NAI_Hull::Maxs( GetHullType() ), MASK_NPCSOLID, this, COLLISION_GROUP_NONE, &tr );

	// If we're stuck in solid, this can't be valid
	if ( tr.allsolid )
	{
		return false;
	}

	// Trace down to find the ground
	AI_TraceHull( tr.endpos, tr.endpos - Vector( 0, 0, 128 ), NAI_Hull::Mins( GetHullType() ), NAI_Hull::Maxs( GetHullType() ), MASK_NPCSOLID, this, COLLISION_GROUP_NONE, &tr );

	// We must end up on the floor with this trace
	if ( tr.fraction < 1.0f )
	{
		result = tr.endpos;
		return true;
	}

	// Ended up in open space
	return false;
}



//-----------------------------------------------------------------------------
// Purpose: 
//
//
// Output : 
//-----------------------------------------------------------------------------
Class_T	CBugNPC::Classify( void )
{
	return	CLASS_SP2_BUG;
}


ConVar	sp2_splash_damage_range( "sp2_splash_damage_range", "1800" );


//-----------------------------------------------------------------------------
// Purpose: 
// Input  : &info - 
//-----------------------------------------------------------------------------
int CBugNPC::OnTakeDamage_Alive( const CTakeDamageInfo &info )
{
	if(m_bIsBurrowed)
		return BaseClass::OnTakeDamage_Alive(info);

	// Calculate the three different colors of damage as a vector
	Vector coloredDamage = info.GetAttacker()->GetDamageColor() * info.GetDamage();
	coloredDamage.y *=  sp2_turret_dmg_multiplier.GetFloat();
	m_vecAccumulatedDamage += coloredDamage;

	//  Adjust damage with the bug's armorcolorfactor
	Vector dealtDamage = coloredDamage * m_vecArmorColor;

	//Msg("Original Damage: %f, Colored damage: %f, %f, %f\n", info.GetDamage(), newDamage.x, newDamage.y, newDamage.z);

	CTakeDamageInfo newInfo = info;

	// Recalculate damage:
	newInfo.SetDamage((dealtDamage.x + dealtDamage.y + dealtDamage.z));

	FindOrAddEntityRelationship( info.GetAttacker() )->priority += newInfo.GetDamage();

	CAI_Network*		ai = g_pBigAINet;
	int nodeId = ai->NearestNodeToPoint(this->GetAbsOrigin(), false);

	CNodeWeight* weighter = g_pAINetworkManager->m_pNodeWeight;
	
	if(nodeId >= 0 && nodeId < weighter->Count()) 
	{
		Vector dangerChange = coloredDamage / sp2_bug_initial_health.GetFloat();

		// Maximum register 1 death each time damage is dealt
		dangerChange.x = min(dangerChange.x, 1);
		dangerChange.y = min(dangerChange.y, 1);
		dangerChange.z = min(dangerChange.z, 1);

		weighter->ChangeWeight(nodeId, dangerChange);

		float range = sp2_splash_damage_range.GetFloat();

		// Give neighbour nodes damage according to their distance to the current node.
		CAI_Node* node =  ai->GetNode(nodeId);
		for(int i=0;i<node->NumLinks();i++) 
		{
			CAI_Link* link =  node->GetLinkByIndex(i);
			CAI_Node* destNode = ai->GetNode(link->m_iDestID);
			float dist = (node->GetOrigin() - destNode->GetOrigin()).Length();
			if(dist < range) 
			{
				float distFactor = 1 - (dist / range);
				weighter->ChangeWeight(link->m_iDestID, dangerChange * distFactor);
			}
		}
	}
	if(newInfo.GetDamage() > 0)
		PainSound( newInfo );


	return BaseClass::OnTakeDamage_Alive(newInfo);
}

void CBugNPC::OnFriendDamaged( CBaseCombatCharacter *pSquadmate, const CTakeDamageInfo &info )
{
	if(m_bIsBurrowed)
		return;
	Relationship_t* r = FindOrAddEntityRelationship( info.GetAttacker() );
	r->priority += info.GetDamage() * 0.75;
	
/*	if(!GetEnemy() || FindOrAddEntityRelationship( GetEnemy() )->priority < r->priority  )
		SetEnemy(info.GetAttacker());
*/
	BaseClass::OnFriendDamaged( pSquadmate, info );
}

void CBugNPC::Event_Killed( const CTakeDamageInfo &info )
{
	if(this->m_eBRAAIN != NULL)
		m_eBRAAIN->AntKilled(this);

	CPASAttenuationFilter filter( this );

	BaseClass::Event_Killed(info);
	EmitSound( filter, entindex(), "NPC_Antlion.OMG" );

}


//-----------------------------------------------------------------------------
// Purpose: 
//-----------------------------------------------------------------------------
void CBugNPC::IdleSound( void )
{
	//EmitSound( "NPC_ Antlion.Idle" );
}

//-----------------------------------------------------------------------------
// Purpose: 
//-----------------------------------------------------------------------------
void CBugNPC::PainSound( const CTakeDamageInfo &info )
{
	EmitSound( "NPC_Antlion.Pain" );
}


void CBugNPC::BuildScheduleTestBits( void )
{
	if(!IsCurSchedule( SCHED_BUG_BURROW ) && !IsCurSchedule( SCHED_BUG_BURROWED ) )
		SetCustomInterruptCondition( COND_BUG_REACHED_GOAL );
	BaseClass::BuildScheduleTestBits();
}



void CBugNPC::GatherConditions()
{
	if(m_eTargetPOI && m_eTargetPOI->GetAbsOrigin().DistTo(GetAbsOrigin()) < m_eTargetPOI->m_flSpawnRadius  )
	{
		SetCondition( COND_BUG_REACHED_GOAL );
	}

	BaseClass::GatherConditions();
}


void CBugNPC::HandleAnimEvent( animevent_t *pEvent )
{
	if ( pEvent->event == AE_ANTLION_WALK_FOOTSTEP )
	{

		MakeAIFootstepSound( 240.0f );
		EmitSound( "NPC_Antlion.Footstep", m_hFootstep, pEvent->eventtime );
		return;
	}


	if ( pEvent->event == AE_ANTLION_VANISH )
	{
		//AddSolidFlags( FSOLID_NOT_SOLID );
		m_takedamage = DAMAGE_NO;
		AddEffects( EF_NODRAW );
		//SetWings( false );

		return;
	}


	if ( pEvent->event == AE_ANTLION_BURROW_IN )
	{
		//Burrowing sound
		EmitSound( "NPC_Antlion.BurrowIn" );

		//Shake the screen
		UTIL_ScreenShake( GetAbsOrigin(), 12.0f, 127.0f, 1.3f, 1024.0f, SHAKE_START );

		AddSolidFlags( FSOLID_NOT_SOLID );

		//Throw dust up
		CreateDust();

		//if ( GetHintNode() )
		//{
		//	GetHintNode()->Unlock( 2.0f );
		//}

		return;
	}

	if ( pEvent->event == AE_ANTLION_BURROW_OUT )
	{
			EmitSound( "NPC_Antlion.BurrowOut" );

		//Shake the screen
		UTIL_ScreenShake( GetAbsOrigin(), 0.5f, 80.0f, 1.0f, 256.0f, SHAKE_START );

		//Throw dust up
		CreateDust();

		RemoveEffects( EF_NODRAW );
		RemoveFlag( FL_NOTARGET );

		return;
	}

	if ( pEvent->event == AE_ANTLION_FOOTSTEP_SOFT )
	{
		EmitSound( "NPC_Antlion.FootstepSoft", pEvent->eventtime );
		return;
	}

	if ( pEvent->event == AE_ANTLION_FOOTSTEP_HEAVY )
	{
		EmitSound( "NPC_Antlion.FootstepHeavy", pEvent->eventtime );
		return;
	}
	
	
	if ( pEvent->event == AE_ANTLION_MELEE1_SOUND )
	{
		EmitSound( "NPC_Antlion.MeleeAttackSingle" );
		return;
	}
	
	if ( pEvent->event == AE_ANTLION_MELEE2_SOUND )
	{
		EmitSound( "NPC_Antlion.MeleeAttackDouble" );
		return;
	}

	if ( pEvent->event == AE_ANTLION_MELEE_HIT1 )
	{
		MeleeAttack( BUG_MELEE1_RANGE, 25, QAngle( 20.0f, 0.0f, -12.0f ), Vector( -250.0f, 1.0f, 1.0f ) );
		return;
	}

	if ( pEvent->event == AE_ANTLION_MELEE_HIT2 )
	{
		MeleeAttack( BUG_MELEE1_RANGE, 25, QAngle( 20.0f, 0.0f, 0.0f ), Vector( -350.0f, 1.0f, 1.0f ) );
		return;
	}


}

int CBugNPC::SelectSchedule( )
{
	if(m_eBRAAIN == NULL && m_eTargetPOI == NULL) 
	{
		return SCHED_IDLE_STAND;
	}
	if(HasCondition(COND_BUG_REACHED_GOAL))
	{
		//OnTakeDamage( CTakeDamageInfo(this, this, Vector(0,100,0), Vector(this->GetAbsOrigin()), m_iHealth +1, DMG_BLAST));
		if(!m_bIsBurrowed)
			return SCHED_BUG_BURROW;
		else {
			m_spawnflags |= SF_NPC_GAG;
			//OnTakeDamage( CTakeDamageInfo(this, this, Vector(0,100,0), Vector(this->GetAbsOrigin()), m_iHealth +1, DMG_REMOVENORAGDOLL | DMG_BLAST));
			RemoveDeferred();
			if(m_eBRAAIN != NULL && m_eTargetPOI != NULL) {
				m_eBRAAIN->ReachedPOI(this, m_eTargetPOI);
				m_eTargetPOI = NULL;
			}
			return SCHED_BUG_BURROWED;
		}
	}

	if(GetEnemy() != NULL) 
	{
		//Msg("HATE: %i \n", FindEntityRelationship( GetEnemy() )->priority  );
		if(FindOrAddEntityRelationship( GetEnemy() )->priority > 20)
			return BaseClass::SelectSchedule();
	}




	return m_bCanRun ? SCHED_RUN_TO_GOAL : SCHED_WALK_TO_GOAL;
}

ConVar	sp2_danger_level_cap( "sp2_danger_level_cap", "10" );
ConVar	sp2_danger_cost( "sp2_danger_cost", "2400" );

bool CBugNPC::MovementCost(int movetype, const Vector &vecStart, const Vector &vecEnd, float *pCost, CAI_Node* start, CAI_Node* end)
{
	int eI = 0;
	/*if(start != NULL && start->GetId() < g_pAINetworkManager->m_pNodeWeight->Count()) 
	{
		//fStartWeight = g_pAINetworkManager->m_pNodeWeight->GetNodeWeight(start->GetId())  .Length();
		sI = start->GetId();
	}*/
	
	if(end != NULL && end->GetId() < g_pAINetworkManager->m_pNodeWeight->Count()) 
	{
		Vector weight = g_pAINetworkManager->m_pNodeWeight->GetNodeWeight(end->GetId()) * m_vecCostFactor;

		float additional = min(weight.x + weight.y + weight.z, sp2_danger_level_cap.GetFloat());

		if(additional > 0)
			*pCost += additional * sp2_danger_cost.GetFloat();
		
		eI = end->GetId();
	}

	//Msg("Cost: %i -> %i = %.1f, Weights: %.1f -> %.1f \n", sI, eI, *pCost, fStartWeight, fEndWeight );
	return true;
}

bool CBugNPC::ShouldGib( const CTakeDamageInfo &info ) 
{ 
	return sp2_bug_always_gib.GetBool() || m_vecAccumulatedDamage.x > m_vecAccumulatedDamage.y; 
}	// Always ragdoll, unless specified by the leaf class


//-----------------------------------------------------------------------------
// Purpose:  
//-----------------------------------------------------------------------------
void CBugNPC::StartTask( const Task_t *pTask )
{
	switch ( pTask->iTask ) 
	{
	case TASK_FIND_BUG_GOAL:
		{
			if(m_eTargetPOI == NULL) {
				TaskFail(FAIL_NO_ROUTE);
				break;
			}

			Vector v = m_eTargetPOI->GetAbsOrigin();

			TranslateNavGoal( m_eTargetPOI, v );

			if (!GetNavigator()->SetGoal( v ))
			{
				TaskFail(FAIL_NO_ROUTE);
			}
			else
			{
				GetNavigator()->SetGoalTolerance( 48 );
			}


			break;
		}
	case TASK_STAY_BURROWED:
		m_bIsBurrowed = true;
		AddEffects( EF_NODRAW );
		TaskComplete();
		break;

	default:
		BaseClass::StartTask( pTask );
		break;
	}
}


//-----------------------------------------------------------------------------
// Purpose: 
// Input  : *pTask - 
//-----------------------------------------------------------------------------
void CBugNPC::RunTask( const Task_t *pTask )
{
	switch ( pTask->iTask )
	{

	default:
		BaseClass::RunTask( pTask );
		break;
	}
}


int CBugNPC::MeleeAttack1Conditions( float flDot, float flDist )
{

	float		flPrDist, flPrDot;
	Vector		vecPrPos;
	Vector2D	vec2DPrDir;

	//Get our likely position in one half second
	UTIL_PredictedPosition( GetEnemy(), 0.5f, &vecPrPos );

	//Get the predicted distance and direction
	flPrDist	= ( vecPrPos - GetAbsOrigin() ).Length();
	vec2DPrDir	= ( vecPrPos - GetAbsOrigin() ).AsVector2D();

	Vector vecBodyDir = BodyDirection2D();

	Vector2D vec2DBodyDir = vecBodyDir.AsVector2D();
	
	flPrDot	= DotProduct2D ( vec2DPrDir, vec2DBodyDir );

	trace_t	tr;
	AI_TraceHull( WorldSpaceCenter(), GetEnemy()->WorldSpaceCenter(), -Vector(8,8,8), Vector(8,8,8), MASK_SOLID_BRUSHONLY, this, COLLISION_GROUP_NONE, &tr );

	if ( tr.m_pEnt != GetEnemy() && tr.fraction < 1.0f )
		return 0;

	if ( flPrDist > BUG_MELEE1_RANGE )
		return COND_TOO_FAR_TO_ATTACK;

	if ( flPrDot < 0.5f )
		return COND_NOT_FACING_ATTACK;

	return COND_CAN_MELEE_ATTACK1;
}

void CBugNPC::MeleeAttack( float distance, float damage, QAngle &viewPunch, Vector &shove )
{
	Vector vecForceDir;

	CBaseEntity *pHurt = CheckTraceHullAttack( distance, -Vector(16,16,32), Vector(16,16,32), damage, DMG_SLASH, 5.0f );

	if ( pHurt )
	{
		vecForceDir = ( pHurt->WorldSpaceCenter() - WorldSpaceCenter() );


		CBasePlayer *pPlayer = ToBasePlayer( pHurt );

		if ( pPlayer != NULL )
		{
			//Kick the player angles
			if ( !(pPlayer->GetFlags() & FL_GODMODE ) && pPlayer->GetMoveType() != MOVETYPE_NOCLIP )
			{
				pPlayer->ViewPunch( viewPunch );

				Vector	dir = pHurt->GetAbsOrigin() - GetAbsOrigin();
				VectorNormalize(dir);

				QAngle angles;
				VectorAngles( dir, angles );
				Vector forward, right;
				AngleVectors( angles, &forward, &right, NULL );

				//Push the target back
				pHurt->ApplyAbsVelocityImpulse( - right * shove[1] - forward * shove[0] );
			}
		}

		// Play a random attack hit sound
		EmitSound( "NPC_Antlion.MeleeAttack" );
	}
}

void CBugNPC::SetTargetPOI( BugPOI *targetPOI )
{
	m_eTargetPOI = targetPOI;
}

// Purpose: 
// Output : Returns true on success, false on failure.
//-----------------------------------------------------------------------------
bool CBugNPC::CorpseGib( const CTakeDamageInfo &info )
{
	CEffectData	data;

	data.m_vOrigin = WorldSpaceCenter();
	data.m_vNormal = data.m_vOrigin - info.GetDamagePosition();
	VectorNormalize( data.m_vNormal );

	data.m_flScale = RemapVal( m_iHealth, 0, -500, 1, 3 );
	data.m_flScale = clamp( data.m_flScale, 1, 3 );

	DispatchEffect( "AntlionGib", data );

	//CSoundEnt::InsertSound( SOUND_PHYSICS_DANGER, GetAbsOrigin(), 256, 0.5f, this );

	return true;
}


//-----------------------------------------------------------------------------
// Purpose: 
//-----------------------------------------------------------------------------
void CBugNPC::Burrow( void )
{
	//SetWings( false );

	//Stop us from taking damage and being solid
	m_spawnflags |= SF_NPC_GAG;
}

//-----------------------------------------------------------------------------
// Purpose: 
//-----------------------------------------------------------------------------
void CBugNPC::Unburrow( void )
{
	m_bIsBurrowed = false;
	//SetWings( false );

	//Become solid again and visible
	m_spawnflags &= ~SF_NPC_GAG;
	RemoveSolidFlags( FSOLID_NOT_SOLID );
	m_takedamage	= DAMAGE_YES;

	SetGroundEntity( NULL );

	//If we have an enemy, come out facing them
	if ( GetEnemy() )
	{
		Vector	dir = GetEnemy()->GetAbsOrigin() - GetAbsOrigin();
		VectorNormalize(dir);

		QAngle angles = GetAbsAngles();
		angles[ YAW ] = UTIL_VecToYaw( dir );
		SetLocalAngles( angles );
	}
}

//-----------------------------------------------------------------------------
// Purpose: 
//-----------------------------------------------------------------------------
void CBugNPC::CreateDust( bool placeDecal )
{
	trace_t	tr;
	AI_TraceLine( GetAbsOrigin()+Vector(0,0,1), GetAbsOrigin()-Vector(0,0,64), MASK_SOLID_BRUSHONLY | CONTENTS_PLAYERCLIP | CONTENTS_MONSTERCLIP, this, COLLISION_GROUP_NONE, &tr );

	if ( tr.fraction < 1.0f )
	{
		surfacedata_t *pdata = physprops->GetSurfaceData( tr.surface.surfaceProps );

		if ( hl2_episodic.GetBool() == true || ( pdata->game.material == CHAR_TEX_CONCRETE ) || 
			( pdata->game.material == CHAR_TEX_DIRT ) ||
			( pdata->game.material == CHAR_TEX_SAND ) ) 
		{

			UTIL_CreateAntlionDust( tr.endpos + Vector(0,0,24), GetAbsAngles() );

			if ( placeDecal )
			{
				UTIL_DecalTrace( &tr, "Antlion.Unburrow" );
			}
		}
	}
}
